package JoeDog::ProgressBar;

use vars qw($VERSION %DEFAULT);
use strict;

$VERSION = '1.1';

%DEFAULT = (
  freq   => 1,
  pct    => 0,
  lst    => 0,
  now    => 0,
);

# Based on AsciiProgressBar by Richard Sands
# Original code base: Jan 16 2001, 1:40 pm

=head1 SYNOPSIS

  JoeDog::ProgressBar - A Perl extension for ASCII progress bars

  use JoeDog::ProgressBar;
  my $bar = new JoeDog::ProgressBar();
  my $max = 200000;

  for(0..$max){
    $bar->refresh($_);
  }

=head1 METHODS

=head2 new

  ProgressBar constructor: returns a reference to a 
  JoeDog::ProgressBar object.

  example: $bar = new JoeDog::ProgressBar($max);

=cut

sub new {
  my ($this, $max) = @_; 
  my $class        = ref($this) || $this;
  my $self         = {};

  $self->{max}  = $max || die("JoeDog::ProgressBar requires a range ceiling");
  $self->{fg}   = '+';
  $self->{bg}   = ' ';
  $self->{ptr}  = '>';
  
  bless($self, $class);
  $self->init();

  return($self);
}

sub init
{
  my $this = shift;

  foreach my $key (keys %DEFAULT){
    $this->{$key} = $DEFAULT{$key};
  }
  return;
}

=head2 set

  instance method: allows you to set values for 
  configurable parameters:
  
  fg  = the foreground character in the bar (default: '=')
  bg  = the background character in the bar (default: ' ')
  ptr = the bar's pointer character (default: '>') 

  example:
  $bar = new JoeDog::ProgressBar($max);
  $bar->set(
  {
    fg   => '=',
    bg   => '#',
    ptr  => '*',
  }) or die "unable to set attributes"; 

=cut
sub set
{
  my $this = shift;
  my %args = %{$_[0]}; 
  my $i    = 0;

  foreach my $k (keys %args){
    next if($this->_has_default($k));
    $this->{$k} = $args{$k}; 
    $i++;
  }

  return $i;
}

=head2 refresh

  instance method: allows you to iterate the progress bar
  example:
  my $cnt = 1;
  my $max = scalar keys %hash;
  my $bar = JoeDog::ProgressBar($max);
  for my $key (keys %hash){
    # do something ... 
 
    $bar->refresh($cnt++);
  } 

=cut

sub refresh 
{
  my $this     = shift;
  $this->{now} = shift;
  unless( defined $this->{now} and $this->{now} >= 0 ){
    die('method "refresh" missing / invalid argument');
  }

  $this->{pct} = $this->{freq} * int((($this->{now} * 100) / $this->{max} ) / $this->{freq});
  if($this->{pct} > $this->{lst}){
    print(
      "\r", sprintf("%3s", int $this->{pct}), "% done: ", $this->{fg} x int($this->{pct} / 2), 
      sprintf("%s", ($this->{pct} != 100 ? $this->{ptr} : "|")), 
      $this->{ bg } x ( 50 - int( $this->{ pct } / 2 ) )
    );
    $this->{lst} = $this->{pct};
  }
}

sub _has_default
{
  my $this = shift;
  my $attr = shift; 
  return exists $DEFAULT{$attr};
}

1; 

__END__

=head1 DESCRIPTION

JoeDog::ProgressBar is a module that provides the programmer with a
ASCII text progress bar in perl modules.

=head1 AUTHOR

Jeffrey Fulmer, jeff@joedog.org

=head1 SEE ALSO

perl(1).

=cut
